/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.gui;

import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;

import org.joda.time.DateTime;

import eu.agrosense.client.calendar.api.event.BarEvent;
import eu.agrosense.client.calendar.api.event.BarEventHandler;
import eu.agrosense.client.calendar.api.model.CalendarNode;

//TODO styles
public class Bar extends Group {

    public static final int BAR_HEIGHT = 50;
    private RegionWithTitle bar;
    private String seriesStyleClass;
    private String dataStyleClass;
    private Tooltip tooltip;
    private int depth; 
    private Label label = new Label();
    private int order = 0;
    private BarEventHandler onBarClicked;
    private CalendarNode node;
    
    public Bar(CalendarNode node, String seriesStyleClass, String dataStyleClass, int depth) {
        this.node = node;
        this.depth = depth;
        this.seriesStyleClass = seriesStyleClass;
        this.dataStyleClass = dataStyleClass;

        setAutoSizeChildren(false);
        bar = new RegionWithTitle(node.getName());
        getChildren().addAll(bar);
        updateStyleClasses();
        tooltip = new Tooltip();
        tooltip.setGraphic(new TooltipContent());
        Tooltip.install(bar, tooltip);

        setCursor(javafx.scene.Cursor.HAND);
        initListeners();
    }

    private void initListeners() {
        bar.setOnMouseClicked(new EventHandler<MouseEvent>() {
            @Override
            public void handle(MouseEvent mouseEvent) {
                barClicked(mouseEvent);
            }
        });

        setOnMouseEntered(new EventHandler<MouseEvent>() {

			@Override
            public void handle(MouseEvent mouseEvent) {
                sendToFront();
                setMouseOverStyle();
            }
        });

        setOnMouseExited(new EventHandler<MouseEvent>() {
        	
            @Override
            public void handle(MouseEvent mouseEvent) {
                sendToBack();
                setStandardStyle();
            }
        });
        
       
    }

    public void setSeriesAndDataStyleClasses(String seriesStyleClass, String dataStyleClass) {
        this.seriesStyleClass = seriesStyleClass;
        this.dataStyleClass = dataStyleClass;
        updateStyleClasses();
    }

    public void update(double width, int depth) {
        this.depth = depth;
        updateStyleClasses();
        resize(width, BAR_HEIGHT);
        bar.resize(width, BAR_HEIGHT);
    }

    public void updateTooltip(String title, String category, DateTime dateFrom, DateTime dateTo) {
        TooltipContent tooltipContent = (TooltipContent) tooltip.getGraphic();
        tooltipContent.update(title, dateFrom, dateTo);
        label.setText(title);
    }

    private void updateStyleClasses() {
        getStyleClass().setAll("calendar-bar", seriesStyleClass, dataStyleClass);
        setStandardStyle();
    }
 
    private void setStandardStyle() {
        bar.getStyleClass().setAll("calendar-bar", seriesStyleClass, dataStyleClass, getStylePostfix(depth)); 
    }

    private void setMouseOverStyle() {
        bar.getStyleClass().setAll("calendar-bar", seriesStyleClass, dataStyleClass, "mouse-over");
    }

    public int getOrder() {
        return order;
    }

    public void setOrder(int order) {
        this.order = order;
    }

    private CalendarChart getChartParent() {
        Node parent = getParent();
        while (parent != null) {
            if (parent instanceof CalendarChart) {
                return (CalendarChart) parent;
            } else {
                parent = parent.getParent();
            }
        }

        return null;
    }

    private void sendToFront() {
        CalendarChart calendarChart = getChartParent();
        if (calendarChart != null) {
            calendarChart.toFront(this);
        }
    }

    private void sendToBack() {
        CalendarChart calendarChart = getChartParent();
        if (calendarChart != null) {
            calendarChart.toBack(this);
        }

    }

    private String getStylePostfix(int depth) {
        if (depth == 0) {
            return "master";
        } else if (depth == 1) {
            return "child_0"; //depth 1 is treated specially to enable circular colors
        } else {
            return "child_" + (((depth - 2) % 4) + 1);
        }
    }

	private void barClicked(MouseEvent event) {
		MouseButton mouseButton = event.getButton();
		if (MouseButton.PRIMARY.equals(mouseButton) && event.isControlDown()) {
			onBarClicked.handle(new BarEvent(node));
		}
	}

    public void setOnBarClicked(BarEventHandler onBarClicked) {
        this.onBarClicked = onBarClicked;
    }

    public BarEventHandler getOnBarClicked() {
        return onBarClicked;
    }
    
	public void updateLayout(double xStart, double yStart) {
		setLayoutX(xStart);
		setLayoutY(yStart);
		bar.updateTitleLayout(xStart, bar.getLayoutBounds().getWidth());
	}
    
}
