/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.internal.util;

import java.util.ArrayList;
import java.util.List;
import org.joda.time.DateTime;

import eu.agrosense.client.calendar.api.model.CalendarExtras;
import eu.agrosense.client.calendar.api.model.CalendarNode;

/**
 * Turning tree node model to list of nodes.
 * 
 * @author Frantisek Post
 *
 */
public class ModelInflater {

	private List<DateTime> dates = new ArrayList<>();
	private List<CalendarExtras> extras = new ArrayList<>();
	private List<CalendarNode> nodes = new ArrayList<>();
	private CalendarNode node;
	
	public ModelInflater(CalendarNode node) {
		this.node = node;
		inflateModel();
	}
	
	protected void inflateModel() {
		inflateInternal(node);
	}
	
	protected void inflateInternal(CalendarNode node) {
		dates.add(node.getStart());
		
		int depth = calculateDepth(node);
		CalendarExtras extra = new CalendarExtras(node.getEnd(), node.getName(), depth);
		extras.add(extra);
		
		nodes.add(node);
		
		for (CalendarNode child : node.getChildren()) {
			inflateInternal(child);
		}
	}

	private int calculateDepth(CalendarNode node) {
	    int depth = 0;
	    CalendarNode n = node;
	    while (n.getParent() != null) {
	    	depth++;
	    	n = n.getParent();
	    }
	    return depth;
    }
	
	public DateTime[] getDates() {
	    return dates.toArray(new DateTime[dates.size()]);
    }
	
	public CalendarExtras[] getExtras() {
	    return extras.toArray(new CalendarExtras[extras.size()]);
    }
	
	public List<CalendarNode> getNodes() {
	    return nodes;
    }
	
}
