/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar.api.model;

import java.util.ArrayList;
import java.util.List;
import org.joda.time.DateTime;
import org.joda.time.LocalDate;

import eu.agrosense.client.calendar.api.internal.util.DateUtil;

public class CalendarNode {

    private String name;
    private DateTime start;
    private DateTime end;
    private List<CalendarNode> children = new ArrayList<>();
    private CalendarNode parent;

    public CalendarNode(String name, DateTime start, DateTime end) {
        super();
        this.name = name;
        this.start = start;
        this.end = end;
    }

    public CalendarNode(String name, LocalDate start, LocalDate end) {
        super();
        this.name = name;
        this.start = DateUtil.startOfDay(start);
        this.end = DateUtil.endOfDay(end);
    }

    public DateTime getStart() {
        return start;
    }

    public DateTime getEnd() {
        return end;
    }

    public CalendarNode getParent() {
        return parent;
    }

    void setParent(CalendarNode parent) {
        this.parent = parent;
    }

    public void addChild(CalendarNode node) {
        if (node.getParent() != null) {
            throw new IllegalStateException("model already has a parent");
        }
        children.add(node);
        node.setParent(this);
    }

    public void removeChild(CalendarNode node) {
        if (node.getParent() != this) {
            throw new IllegalStateException("doesn't contain this model");
        }
        children.remove(node);
        node.setParent(null);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    //TODO immutable?
    public List<CalendarNode> getChildren() {
        return children;
    }
}
